@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 // Header that sets HAVE_m128/HAVE_m256 if vector types (__m128/__m256) are available
@@ -28,11 +24,17 @@
 //  did not always provide proper stack alignment in destructors of such objects.
 
 #if (_MSC_VER>=1600)
-//TODO: handle /arch:AVX in the right way. 
-#pragma warning (push)
-#pragma warning (disable: 4752)
+//TODO: handle /arch:AVX in the right way.
+// #pragma warning (push)
+// #pragma warning (disable: 4752)
 #endif
 
+#if __TBB_GCC_WARNING_SUPPRESSION_PRESENT && __TBB_GCC_WARNING_IGNORED_ATTRIBUTES_PRESENT
+// #pragma GCC diagnostic push
+// #pragma GCC diagnostic ignored "-Wignored-attributes"
+#endif
+
+
 template<typename __Mvec>
 class ClassWithVectorType {
     static const int n = 16;
@@ -40,7 +42,7 @@ class ClassWithVectorType {
     __Mvec field[n];
     void init( int start );
 public:
-    ClassWithVectorType() {init(-n);} 
+    ClassWithVectorType() {init(-n);}
     ClassWithVectorType( int i ) {init(i);}
     void operator=( const ClassWithVectorType& src ) {
         __Mvec stack[n];
@@ -65,7 +67,7 @@ template<typename __Mvec>
 void ClassWithVectorType<__Mvec>::init( int start ) {
     __Mvec stack[n];
     for( int i=0; i<n; ++i ) {
-        // Declaring value as a one-element array instead of a scalar quites 
+        // Declaring value as a one-element array instead of a scalar quites
         // gratuitous warnings about possible use of "value" before it was set.
         __Mvec value[1];
         for( int j=0; j<F; ++j )
@@ -109,6 +111,10 @@ bool have_AVX() {
 typedef ClassWithVectorType<__m128> ClassWithSSE;
 #endif
 
+#if __TBB_GCC_WARNING_SUPPRESSION_PRESENT && __TBB_GCC_WARNING_IGNORED_ATTRIBUTES_PRESENT
+// #pragma GCC diagnostic pop
+#endif
+
 #if (_MSC_VER>=1600)
-#pragma warning (pop)
+// #pragma warning (pop)
 #endif
