@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #define NOMINMAX
@@ -25,6 +21,8 @@
 #include "harness.h"
 #include "harness_allocator.h"
 
+using tbb::internal::spin_wait_while;
+
 #include <vector>
 
 static tbb::atomic<long> FooConstructed;
@@ -76,7 +74,7 @@ static const int N = 50000;     // # of bytes
 //! Exception for concurrent_queue
 class Foo_exception : public std::bad_alloc {
 public:
-    virtual const char *what() const throw() { return "out of Foo limit"; }
+    virtual const char *what() const throw() __TBB_override { return "out of Foo limit"; }
     virtual ~Foo_exception() throw() {}
 };
 
@@ -165,7 +163,7 @@ struct Body: NoAssign {
     void operator()( int thread_id ) const {
         long pop_kind[3] = {0,0,0};
         int serial[MAXTHREAD+1];
-        memset( serial, 0, nthread*sizeof(int) );
+        memset( static_cast<void*>(serial), 0, nthread*sizeof(int) );
         ASSERT( thread_id<nthread, NULL );
 
         long sum = 0;
@@ -520,7 +518,7 @@ bool operator!=(const BarIterator& bia, const BarIterator& bib) {
 #if TBB_USE_EXCEPTIONS
 class Bar_exception : public std::bad_alloc {
 public:
-    virtual const char *what() const throw() { return "making the entry invalid"; }
+    virtual const char *what() const throw() __TBB_override { return "making the entry invalid"; }
     virtual ~Bar_exception() throw() {}
 };
 
@@ -713,7 +711,7 @@ void TestConstructors ()
 #endif /* TBB_USE_EXCEPTIONS */
 
 #if __TBB_CPP11_RVALUE_REF_PRESENT
-    // Testing work of move constructors
+    // Testing work of move constructors. TODO: merge into TestMoveConstructors?
     src_queue.clear();
 
     typedef typename CQ::size_type qsize_t;
@@ -788,24 +786,25 @@ void TestMoveConstructors() {
     const size_t size = 10;
     for( size_t i = 0; i < size; ++i )
         src_queue.push( T(i + (i ^ size)) );
-
     ASSERT( T::construction_num == 2 * size, NULL );
     ASSERT( T::destruction_num == size, NULL );
+
     const T* locations[size];
     typename CQ::const_iterator qit = src_queue.unsafe_begin();
     for( size_t i = 0; i < size; ++i, ++qit )
         locations[i] = &(*qit);
 
     // Ensuring allocation operation takes place during move when allocators are different
+    T::construction_num = T::destruction_num = 0;
     CQ dst_queue( std::move(src_queue), allocator<T>(1) );
-    ASSERT( T::construction_num == 2 * size + size, NULL );
-    ASSERT( T::destruction_num == 2 * size + size, NULL );
+    ASSERT( T::construction_num == size, NULL );
+    ASSERT( T::destruction_num == size+1, NULL ); // One item is used by the queue destructor
 
     TestQueueOperabilityAfterDataMove<T>( src_queue );
 
     qit = dst_queue.unsafe_begin();
     for( size_t i = 0; i < size; ++i, ++qit ) {
-        ASSERT( locations[i] != &(*qit), "item was not moved" );
+        ASSERT( locations[i] != &(*qit), "an item should have been copied but was not" );
         locations[i] = &(*qit);
     }
 
@@ -819,7 +818,7 @@ void TestMoveConstructors() {
 
     qit = dst_queue2.unsafe_begin();
     for( size_t i = 0; i < size; ++i, ++qit ) {
-        ASSERT( locations[i] == &(*qit), "item was moved" );
+        ASSERT( locations[i] == &(*qit), "an item should have been moved but was not" );
     }
 
     for( size_t i = 0; i < size; ++i) {
@@ -837,34 +836,34 @@ void TestMoveConstruction() {
     REMARK("Testing move constructors with specified allocators...");
     TestMoveConstructors< ConcQWithSizeWrapper< Bar, allocator<Bar> >, Bar >();
     TestMoveConstructors< tbb::concurrent_bounded_queue< Bar, allocator<Bar> >, Bar >();
+    // TODO: add tests with movable data
     REMARK(" work\n");
 }
 #endif /* __TBB_CPP11_RVALUE_REF_PRESENT */
 
 template<typename Iterator1, typename Iterator2>
 void TestIteratorAux( Iterator1 i, Iterator2 j, int size ) {
-    // Now test iteration
-    Iterator1 old_i;
+    Iterator1 old_i; // assigned at first iteration below
     for( int k=0; k<size; ++k ) {
         ASSERT( i!=j, NULL );
         ASSERT( !(i==j), NULL );
-        Foo f;
+        // Test "->"
+        ASSERT( k+1==i->serial, NULL );
         if( k&1 ) {
-            // Test pre-increment
-            f = *old_i++;
+            // Test post-increment
+            Foo f = *old_i++;
+            ASSERT( k+1==f.serial, NULL );
             // Test assignment
             i = old_i;
         } else {
-            // Test post-increment
-            f=*i++;
+            // Test pre-increment
             if( k<size-1 ) {
-                // Test "->"
-                ASSERT( k+2==i->serial, NULL );
-            }
+                Foo f = *++i;
+                ASSERT( k+2==f.serial, NULL );
+            } else ++i;
             // Test assignment
             old_i = i;
         }
-        ASSERT( k+1==f.serial, NULL );
     }
     ASSERT( !(i!=j), NULL );
     ASSERT( i==j, NULL );
@@ -1025,7 +1024,7 @@ void TestNegativeQueue( int nthread ) {
 }
 
 #if TBB_USE_EXCEPTIONS
-template<typename CQ,typename A1,typename A2,typename T>
+template<template<typename, typename> class CQ,typename A1,typename A2,typename T>
 void TestExceptionBody() {
     enum methods {
         m_push = 0,
@@ -1038,11 +1037,12 @@ void TestExceptionBody() {
     // Do test on queues of two different types at the same time to
     // catch problem with incorrect sharing between templates.
     {
-        CQ queue0;
-        tbb::concurrent_queue<int,A1> queue1;
+        CQ<T,A1> queue0;
+        CQ<int,A1> queue1;
         for( int i=0; i<2; ++i ) {
             bool caught = false;
             try {
+                // concurrent_queue internally rebinds the allocator to the one for 'char'
                 A2::init_counters();
                 A2::set_limits(N/2);
                 for( int k=0; k<N; k++ ) {
@@ -1063,9 +1063,9 @@ void TestExceptionBody() {
         int n_pushed=0, n_popped=0;
         for(int t = 0; t <= 1; t++)// exception type -- 0 : from allocator(), 1 : from Foo's constructor
         {
-            CQ queue_test;
+            CQ<T,A1> queue_test;
             for( int m=m_push; m<=m_pop; m++ ) {
-                // concurrent_queue internally rebinds the allocator to one with 'char'
+                // concurrent_queue internally rebinds the allocator to the one for 'char'
                 A2::init_counters();
 
                 if(t) MaxFooCount = MaxFooCount + 400;
@@ -1074,64 +1074,62 @@ void TestExceptionBody() {
                 try {
                     switch(m) {
                     case m_push:
-                            for( int k=0; k<N; k++ ) {
-                                push( queue_test, T(), k );
-                                n_pushed++;
-                            }
-                            break;
+                        for( int k=0; k<N; k++ ) {
+                            push( queue_test, T(), k );
+                            n_pushed++;
+                        }
+                        break;
                     case m_pop:
-                            n_popped=0;
-                            for( int k=0; k<n_pushed; k++ ) {
-                                T elt;
-                                queue_test.try_pop( elt );
-                                n_popped++;
-                            }
-                            n_pushed = 0;
-                            A2::set_limits();
-                            break;
+                        n_popped=0;
+                        for( int k=0; k<n_pushed; k++ ) {
+                            T elt;
+                            queue_test.try_pop( elt );
+                            n_popped++;
+                        }
+                        n_pushed = 0;
+                        A2::set_limits();
+                        break;
                     }
                     if( !t && m==m_push ) ASSERT(false, "should throw an exception");
                 } catch ( Foo_exception & ) {
+                    long tc = MaxFooCount;
+                    MaxFooCount = 0; // disable exception
                     switch(m) {
-                    case m_push: {
-                                ASSERT( ptrdiff_t(queue_test.size())==n_pushed, "incorrect queue size" );
-                                long tc = MaxFooCount;
-                                MaxFooCount = 0;
-                                for( int k=0; k<(int)tc; k++ ) {
-                                    push( queue_test, T(), k );
-                                    n_pushed++;
-                                }
-                                MaxFooCount = tc;
-                            }
-                            break;
+                    case m_push:
+                        ASSERT( ptrdiff_t(queue_test.size())==n_pushed, "incorrect queue size" );
+                        for( int k=0; k<(int)tc; k++ ) {
+                            push( queue_test, T(), k );
+                            n_pushed++;
+                        }
+                        break;
                     case m_pop:
-                            MaxFooCount = 0; // disable exception
-                            n_pushed -= (n_popped+1); // including one that threw an exception
-                            ASSERT( n_pushed>=0, "n_pushed cannot be less than 0" );
-                            for( int k=0; k<1000; k++ ) {
-                                push( queue_test, T(), k );
-                                n_pushed++;
-                            }
-                            ASSERT( !queue_test.empty(), "queue must not be empty" );
-                            ASSERT( ptrdiff_t(queue_test.size())==n_pushed, "queue size must be equal to n pushed" );
-                            for( int k=0; k<n_pushed; k++ ) {
-                                T elt;
-                                queue_test.try_pop( elt );
-                            }
-                            ASSERT( queue_test.empty(), "queue must be empty" );
-                            ASSERT( queue_test.size()==0, "queue must be empty" );
-                            break;
+                        n_pushed -= (n_popped+1); // including one that threw the exception
+                        ASSERT( n_pushed>=0, "n_pushed cannot be less than 0" );
+                        for( int k=0; k<1000; k++ ) {
+                            push( queue_test, T(), k );
+                            n_pushed++;
+                        }
+                        ASSERT( !queue_test.empty(), "queue must not be empty" );
+                        ASSERT( ptrdiff_t(queue_test.size())==n_pushed, "queue size must be equal to n pushed" );
+                        for( int k=0; k<n_pushed; k++ ) {
+                            T elt;
+                            queue_test.try_pop( elt );
+                        }
+                        ASSERT( queue_test.empty(), "queue must be empty" );
+                        ASSERT( queue_test.size()==0, "queue must be empty" );
+                        break;
                     }
+                    MaxFooCount = tc;
                 } catch ( std::bad_alloc & ) {
                     A2::set_limits(); // disable exception from allocator
                     size_t size = queue_test.size();
                     switch(m) {
                     case m_push:
-                            ASSERT( size>0, "incorrect queue size");
-                            break;
+                        ASSERT( size>0, "incorrect queue size");
+                        break;
                     case m_pop:
-                            if( !t ) ASSERT( false, "should not throw an exceptin" );
-                            break;
+                        if( !t ) ASSERT( false, "should not throw an exception" );
+                        break;
                     }
                 }
                 REMARK("... for t=%d and m=%d, exception test passed\n", t, m);
@@ -1149,8 +1147,8 @@ void TestExceptions() {
 #elif TBB_USE_EXCEPTIONS
     typedef static_counting_allocator<std::allocator<FooEx>, size_t> allocator_t;
     typedef static_counting_allocator<std::allocator<char>, size_t> allocator_char_t;
-    TestExceptionBody<ConcQWithSizeWrapper<FooEx, allocator_t>,allocator_t,allocator_char_t,FooEx>();
-    TestExceptionBody<tbb::concurrent_bounded_queue<FooEx, allocator_t>,allocator_t,allocator_char_t,FooEx>();
+    TestExceptionBody<ConcQWithSizeWrapper,allocator_t,allocator_char_t,FooEx>();
+    TestExceptionBody<tbb::concurrent_bounded_queue,allocator_t,allocator_char_t,FooEx>();
 #endif /* TBB_USE_EXCEPTIONS */
 }
 
@@ -1236,7 +1234,7 @@ void TestFullness()
     TestFullQueue<tbb::concurrent_bounded_queue<Foo>,Foo>();
 }
 
-void TestClearWorks() 
+void TestClearWorks()
 {
     REMARK(" Test concurrent_queue::clear() works\n");
     TestClear<ConcQWithCapacity<Foo> >();
@@ -1264,7 +1262,7 @@ void TestQueueIteratorWorks()
 #endif
 class Empty;
 
-void TestQueueConstructors() 
+void TestQueueConstructors()
 {
     REMARK(" Test concurrent_queue's constructors work\n");
     TestConstructors<ConcQWithSizeWrapper<Bar>,Bar,BarIterator,ConcQWithSizeWrapper<BAR_EX>,BAR_EX>();
@@ -1333,9 +1331,12 @@ class SimplePushBody {
     int max;
 public:
     SimplePushBody(tbb::concurrent_bounded_queue<int>* _q, int hi_thr) : q(_q), max(hi_thr) {}
+    bool operator()() { // predicate for spin_wait_while
+        return q->size()<max;
+    }
     void operator()(int thread_id) const {
         if (thread_id == max) {
-            Harness::Sleep(50);
+            spin_wait_while( *this );
             q->abort();
             return;
         }
@@ -1351,12 +1352,18 @@ public:
 class SimplePopBody {
     tbb::concurrent_bounded_queue<int>* q;
     int max;
+    int prefill;
 public:
-    SimplePopBody(tbb::concurrent_bounded_queue<int>* _q, int hi_thr) : q(_q), max(hi_thr) {}
+    SimplePopBody(tbb::concurrent_bounded_queue<int>* _q, int hi_thr, int nitems)
+    : q(_q), max(hi_thr), prefill(nitems) {}
+    bool operator()() { // predicate for spin_wait_while
+        // There should be `max` pops, and `prefill` should succeed
+        return q->size()>prefill-max;
+    }
     void operator()(int thread_id) const {
         int e;
         if (thread_id == max) {
-            Harness::Sleep(50);
+            spin_wait_while( *this );
             q->abort();
             return;
         }
@@ -1384,8 +1391,14 @@ void TestAbort() {
             NativeParallelFor( nthreads+1, my_push_body1 );
             ASSERT(num_pushed == 0, "no elements should have been pushed to zero-sized queue");
             ASSERT((int)failed_pushes == nthreads, "All threads should have failed to push an element to zero-sized queue");
+            // Do not test popping each time in order to test queue destruction with no previous pops
+            if (nthreads < (MaxThread+MinThread)/2) {
+                int e;
+                bool queue_empty = !iq1.try_pop(e);
+                ASSERT(queue_empty, "no elements should have been popped from zero-sized queue");
+            }
         }
-        
+
         REMARK("...testing pushing to small-sized queue\n");
         tbb::concurrent_bounded_queue<int> iq2;
         iq2.set_capacity(2);
@@ -1394,12 +1407,12 @@ void TestAbort() {
             SimplePushBody my_push_body2(&iq2, nthreads);
             NativeParallelFor( nthreads+1, my_push_body2 );
             ASSERT(num_pushed <= 2, "at most 2 elements should have been pushed to queue of size 2");
-            if (nthreads >= 2) 
+            if (nthreads >= 2)
                 ASSERT((int)failed_pushes == nthreads-2, "nthreads-2 threads should have failed to push an element to queue of size 2");
             int e;
             while (iq2.try_pop(e)) ;
         }
-        
+
         REMARK("...testing popping from small-sized queue\n");
         tbb::concurrent_bounded_queue<int> iq3;
         iq3.set_capacity(2);
@@ -1407,8 +1420,8 @@ void TestAbort() {
             num_pushed = num_popped = failed_pushes = failed_pops = 0;
             iq3.push(42);
             iq3.push(42);
-            SimplePopBody my_pop_body(&iq3, nthreads);
-            NativeParallelFor( nthreads+1, my_pop_body);
+            SimplePopBody my_pop_body(&iq3, nthreads, 2);
+            NativeParallelFor( nthreads+1, my_pop_body );
             ASSERT(num_popped <= 2, "at most 2 elements should have been popped from queue of size 2");
             if (nthreads >= 2)
                 ASSERT((int)failed_pops == nthreads-2, "nthreads-2 threads should have failed to pop an element from queue of size 2");
@@ -1428,10 +1441,10 @@ void TestAbort() {
             SimplePushBody my_push_body2(&iq4, nthreads);
             NativeParallelFor( nthreads+1, my_push_body2 );
             ASSERT((int)num_pushed <= cap, "at most cap elements should have been pushed to queue of size cap");
-            if (nthreads >= cap) 
+            if (nthreads >= cap)
                 ASSERT((int)failed_pushes == nthreads-cap, "nthreads-cap threads should have failed to push an element to queue of size cap");
-            SimplePopBody my_pop_body(&iq4, nthreads);
-            NativeParallelFor( nthreads+1, my_pop_body);
+            SimplePopBody my_pop_body(&iq4, nthreads, (int)num_pushed);
+            NativeParallelFor( nthreads+1, my_pop_body );
             ASSERT((int)num_popped <= cap, "at most cap elements should have been popped from queue of size cap");
             if (nthreads >= cap)
                 ASSERT((int)failed_pops == nthreads-cap, "nthreads-cap threads should have failed to pop an element from queue of size cap");
@@ -1666,9 +1679,41 @@ void TestTypes() {
     TestTypes(arrWk);
 #else
     REPORT("Known issue: C++11 smart pointer tests are skipped.\n");
-#endif /* __TBB_CXX11_TYPES_PRESENT */
+#endif /* __TBB_CPP11_SMART_POINTERS_PRESENT */
 }
 
+#if __TBB_CPP17_DEDUCTION_GUIDES_PRESENT
+template <template <typename...> typename TQueue>
+void TestDeductionGuides() {
+    using ComplexType = const std::string*;
+    std::vector<ComplexType> v;
+
+    // check TQueue(InputIterator, InputIterator)
+    TQueue q1(v.begin(), v.end());
+    static_assert(std::is_same<decltype(q1), TQueue<ComplexType>>::value);
+
+    // check TQueue(InputIterator, InputIterator, Allocator)
+    TQueue q2(v.begin(), v.end(), std::allocator<ComplexType>());
+    static_assert(std::is_same<decltype(q2), TQueue<ComplexType, std::allocator<ComplexType>>>::value);
+
+    // check TQueue(TQueue &)
+    TQueue q3(q1);
+    static_assert(std::is_same<decltype(q3), decltype(q1)>::value);
+
+    // check TQueue(TQueue &, Allocator)
+    TQueue q4(q2, std::allocator<ComplexType>());
+    static_assert(std::is_same<decltype(q4), decltype(q2)>::value);
+
+    // check TQueue(TQueue &&)
+    TQueue q5(std::move(q1));
+    static_assert(std::is_same<decltype(q5), decltype(q1)>::value);
+
+    // check TQueue(TQueue &&, Allocator)
+    TQueue q6(std::move(q4), std::allocator<ComplexType>());
+    static_assert(std::is_same<decltype(q6), decltype(q4)>::value);
+}
+#endif
+
 int TestMain () {
     TestEmptiness();
 
@@ -1703,5 +1748,10 @@ int TestMain () {
 
     TestTypes();
 
+#if __TBB_CPP17_DEDUCTION_GUIDES_PRESENT
+    TestDeductionGuides<tbb::concurrent_queue>();
+    TestDeductionGuides<tbb::concurrent_bounded_queue>();
+#endif
+
     return Harness::Done;
 }
