@@ -1,27 +1,23 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 // Test mixing OpenMP and TBB
 
-/* SCR #471 
- Below is workaround to compile test within enviroment of Intel Compiler
+/* SCR #471
+ Below is workaround to compile test within environment of Intel Compiler
  but by Microsoft Compiler. So, there is wrong "omp.h" file included and
  manifest section is missed from .exe file - restoring here.
 
@@ -40,42 +36,42 @@
 
     #if defined(_DEBUG)
         #if defined(_M_IX86)
-            #pragma comment(linker,"/manifestdependency:\"type='win32' "            \
-                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".DebugOpenMP' "         \
+            #pragma comment(linker,"/manifestdependency:\"type='win32' "        \
+                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".DebugOpenMP' "      \
                 "version='" _CRT_ASSEMBLY_VERSION "' "                          \
                 "processorArchitecture='x86' "                                  \
                 "publicKeyToken='" _VC_ASSEMBLY_PUBLICKEYTOKEN "'\"")
         #elif defined(_M_X64)
-            #pragma comment(linker,"/manifestdependency:\"type='win32' "            \
-                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".DebugOpenMP' "         \
+            #pragma comment(linker,"/manifestdependency:\"type='win32' "        \
+                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".DebugOpenMP' "      \
                 "version='" _CRT_ASSEMBLY_VERSION "' "                          \
-                "processorArchitecture='amd64' "                                  \
+                "processorArchitecture='amd64' "                                \
                 "publicKeyToken='" _VC_ASSEMBLY_PUBLICKEYTOKEN "'\"")
         #elif defined(_M_IA64)
-            #pragma comment(linker,"/manifestdependency:\"type='win32' "            \
-                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".DebugOpenMP' "         \
+            #pragma comment(linker,"/manifestdependency:\"type='win32' "        \
+                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".DebugOpenMP' "      \
                 "version='" _CRT_ASSEMBLY_VERSION "' "                          \
-                "processorArchitecture='ia64' "                                  \
+                "processorArchitecture='ia64' "                                 \
                 "publicKeyToken='" _VC_ASSEMBLY_PUBLICKEYTOKEN "'\"")
         #endif
     #else   // _DEBUG
         #if defined(_M_IX86)
-            #pragma comment(linker,"/manifestdependency:\"type='win32' "            \
-                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".OpenMP' "              \
+            #pragma comment(linker,"/manifestdependency:\"type='win32' "        \
+                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".OpenMP' "           \
                 "version='" _CRT_ASSEMBLY_VERSION "' "                          \
                 "processorArchitecture='x86' "                                  \
                 "publicKeyToken='" _VC_ASSEMBLY_PUBLICKEYTOKEN "'\"")
         #elif defined(_M_X64)
-            #pragma comment(linker,"/manifestdependency:\"type='win32' "            \
-                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".OpenMP' "              \
+            #pragma comment(linker,"/manifestdependency:\"type='win32' "        \
+                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".OpenMP' "           \
                 "version='" _CRT_ASSEMBLY_VERSION "' "                          \
-                "processorArchitecture='amd64' "                                  \
+                "processorArchitecture='amd64' "                                \
                 "publicKeyToken='" _VC_ASSEMBLY_PUBLICKEYTOKEN "'\"")
         #elif defined(_M_IA64)
-            #pragma comment(linker,"/manifestdependency:\"type='win32' "            \
-                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".OpenMP' "              \
+            #pragma comment(linker,"/manifestdependency:\"type='win32' "        \
+                "name='" __LIBRARIES_ASSEMBLY_NAME_PREFIX ".OpenMP' "           \
                 "version='" _CRT_ASSEMBLY_VERSION "' "                          \
-                "processorArchitecture='ia64' "                                  \
+                "processorArchitecture='ia64' "                                 \
                 "publicKeyToken='" _VC_ASSEMBLY_PUBLICKEYTOKEN "'\"")
         #endif
     #endif  // _DEBUG
@@ -90,9 +86,9 @@ typedef short T;
 void SerialConvolve( T c[], const T a[], int m, const T b[], int n ) {
     for( int i=0; i<m+n-1; ++i ) {
         int start = i<n ? 0 : i-n+1;
-        int finish = i<m ? i+1 : m; 
+        int finish = i<m ? i+1 : m;
         T sum = 0;
-        for( int j=start; j<finish; ++j ) 
+        for( int j=start; j<finish; ++j )
             sum += a[j]*b[i-j];
         c[i] = sum;
     }
@@ -111,8 +107,8 @@ using namespace tbb;
 
 #if _MSC_VER && !defined(__INTEL_COMPILER)
     // Suppress overzealous warning about short+=short
-    #pragma warning( push )
-    #pragma warning( disable: 4244 )
+    // #pragma warning( push )
+    // #pragma warning( disable: 4244 )
 #endif
 
 class InnerBody: NoAssign {
@@ -121,34 +117,34 @@ class InnerBody: NoAssign {
     const int i;
 public:
     T sum;
-    InnerBody( T /*c*/[], const T a[], const T b[], int i ) :
-        my_a(a), my_b(b), i(i), sum(0)
+    InnerBody( T /*c*/[], const T a[], const T b[], int ii ) :
+        my_a(a), my_b(b), i(ii), sum(0)
     {}
     InnerBody( InnerBody& x, split ) :
         my_a(x.my_a), my_b(x.my_b), i(x.i), sum(0)
-    { 
+    {
     }
     void join( InnerBody& x ) {sum += x.sum;}
     void operator()( const blocked_range<int>& range ) {
-        for( int j=range.begin(); j!=range.end(); ++j ) 
+        for( int j=range.begin(); j!=range.end(); ++j )
             sum += my_a[j]*my_b[i-j];
     }
 };
 
 #if _MSC_VER && !defined(__INTEL_COMPILER)
-    #pragma warning( pop )
+    // #pragma warning( pop )
 #endif
 
 //! Test OpenMMP loop around TBB loop
 void OpenMP_TBB_Convolve( T c[], const T a[], int m, const T b[], int n ) {
     REMARK("testing OpenMP loop around TBB loop\n");
-#pragma omp parallel 
+#pragma omp parallel
     {
         task_scheduler_init init;
 #pragma omp for
         for( int i=0; i<m+n-1; ++i ) {
             int start = i<n ? 0 : i-n+1;
-            int finish = i<m ? i+1 : m; 
+            int finish = i<m ? i+1 : m;
             InnerBody body(c,a,b,i);
             parallel_reduce( blocked_range<int>(start,finish,10), body );
             c[i] = body.sum;
@@ -163,17 +159,16 @@ class OuterBody: NoAssign {
     const int m;
     const int n;
 public:
-    T sum;
     OuterBody( T c[], const T a[], int m_, const T b[], int n_ ) :
         my_a(a), my_b(b), my_c(c), m(m_), n(n_)
     {}
     void operator()( const blocked_range<int>& range ) const {
         for( int i=range.begin(); i!=range.end(); ++i ) {
             int start = i<n ? 0 : i-n+1;
-            int finish = i<m ? i+1 : m; 
+            int finish = i<m ? i+1 : m;
             T sum = 0;
 #pragma omp parallel for reduction(+:sum)
-            for( int j=start; j<finish; ++j ) 
+            for( int j=start; j<finish; ++j )
                 sum += my_a[j]*my_b[i-j];
             my_c[i] = sum;
         }
@@ -186,29 +181,58 @@ void TBB_OpenMP_Convolve( T c[], const T a[], int m, const T b[], int n ) {
     parallel_for( blocked_range<int>(0,m+n-1,10), OuterBody( c, a, m, b, n ) );
 }
 
-#include <stdio.h>
+#if __INTEL_COMPILER
+// A regression test on OpenMP affinity settings affecting TBB.
+// Testing only with __INTEL_COMPILER because we do not provide interoperability with other OpenMP implementations.
+
+#define __TBB_NUM_THREADS_AFFECTED_BY_LIBIOMP (KMP_VERSION_BUILD<20150922)
+
+void TestNumThreads() {
+#if __TBB_NUM_THREADS_AFFECTED_BY_LIBIOMP
+    REPORT("Known issue: the default number of threads is affected by OpenMP affinity\n");
+#else
+    REMARK("Compare default number of threads for OpenMP and TBB\n");
+    Harness::SetEnv("KMP_AFFINITY","compact");
+    // Make an OpenMP call before initializing TBB
+    int omp_nthreads = omp_get_max_threads();
+    #pragma omp parallel
+    {}
+    int tbb_nthreads = tbb::task_scheduler_init::default_num_threads();
+    REMARK("# of threads (OpenMP): %d\n", omp_nthreads);
+    REMARK("# of threads (TBB): %d\n", tbb_nthreads);
+    // For the purpose of testing, assume that OpenMP and TBB should utilize the same # of threads.
+    // If it's not true on some platforms, the test will need to be adjusted.
+    ASSERT( tbb_nthreads==omp_nthreads, "Initialization of TBB is possibly affected by OpenMP");
+#endif
+}
+#endif // __INTEL_COMPILER
 
 const int M = 17*17;
 const int N = 13*13;
-T a[M], b[N];
+T A[M], B[N];
 T expected[M+N], actual[M+N];
 
 template <class Func>
 void RunTest( Func F, int m, int n, int p, bool wait_workers = false ) {
-    task_scheduler_init init( p, 0, wait_workers );
-    memset( actual, -1, (m+n)*sizeof(T) );
-    F( actual, a, m, b, n );
+    task_scheduler_init init( p );
+    memset( static_cast<void*>(actual), -1, (m+n)*sizeof(T) );
+    F( actual, A, m, B, n );
     ASSERT( memcmp(actual, expected, (m+n-1)*sizeof(T))==0, NULL );
+    if (wait_workers) init.blocking_terminate(std::nothrow);
+    else              init.terminate();
 }
 
 int TestMain () {
+#if __INTEL_COMPILER
+    TestNumThreads(); // Testing initialization-related behavior; must be the first
+#endif // __INTEL_COMPILER
     MinThread = 1;
     for( int p=MinThread; p<=MaxThread; ++p ) {
         for( int m=1; m<=M; m*=17 ) {
             for( int n=1; n<=N; n*=13 ) {
-                for( int i=0; i<m; ++i ) a[i] = T(1+i/5);
-                for( int i=0; i<n; ++i ) b[i] = T(1+i/7);
-                SerialConvolve( expected, a, m, b, n );
+                for( int i=0; i<m; ++i ) A[i] = T(1+i/5);
+                for( int i=0; i<n; ++i ) B[i] = T(1+i/7);
+                SerialConvolve( expected, A, m, B, n );
                 RunTest( OpenMP_TBB_Convolve, m, n, p );
                 RunTest( TBB_OpenMP_Convolve, m, n, p
 #if OPENMP_ASYNC_SHUTDOWN_BROKEN
