@@ -1,29 +1,30 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
+#if __TBB_CPF_BUILD
+#define TBB_DEPRECATED_FLOW_NODE_EXTRACTION 1
+#endif
+
 #include "harness_graph.h"
 
+#include "tbb/flow_graph.h"
 #include "tbb/task_scheduler_init.h"
 
 #define N 300
-#define T 4 
+#define T 4
 #define M 5
 
 template< typename R >
@@ -33,7 +34,7 @@ void simple_read_write_tests() {
 
     for ( int t = 0; t < T; ++t ) {
         R v0(N+1);
-        harness_counting_receiver<R> r[M];
+        std::vector< harness_counting_receiver<R> > r(M, harness_counting_receiver<R>(g));
 
         ASSERT( n.is_valid() == false, NULL );
         ASSERT( n.try_get( v0 ) == false, NULL );
@@ -48,9 +49,9 @@ void simple_read_write_tests() {
            tbb::flow::make_edge( n, r[i] );
         }
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
         ASSERT(n.successor_count() == M, NULL);
-        typename tbb::flow::overwrite_node<R>::successor_vector_type my_succs;
+        typename tbb::flow::overwrite_node<R>::successor_list_type my_succs;
         n.copy_successors(my_succs);
         ASSERT(my_succs.size() == M, NULL);
         ASSERT(n.predecessor_count() == 0, NULL);
@@ -108,7 +109,7 @@ void parallel_read_write_tests() {
 
     for (size_t node_idx=0; node_idx<ow_vec.size(); ++node_idx) {
     for ( int t = 0; t < T; ++t ) {
-        harness_counting_receiver<R> r[M];
+        std::vector< harness_counting_receiver<R> > r(M, harness_counting_receiver<R>(g));
 
         for (int i = 0; i < M; ++i) {
            tbb::flow::make_edge( ow_vec[node_idx], r[i] );
@@ -138,7 +139,7 @@ void parallel_read_write_tests() {
     }
 }
 
-int TestMain() { 
+int TestMain() {
     if( MinThread<1 ) {
         REPORT("number of threads must be positive\n");
         exit(1);
@@ -149,8 +150,9 @@ int TestMain() {
         tbb::task_scheduler_init init(p);
         parallel_read_write_tests<int>();
         parallel_read_write_tests<float>();
+        test_reserving_nodes<tbb::flow::overwrite_node, size_t>();
     }
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
     test_extract_on_node<tbb::flow::overwrite_node, int>();
     test_extract_on_node<tbb::flow::overwrite_node, float>();
 #endif
